from __future__ import annotations

import os
from datetime import date
from pathlib import Path

from skilgen.agents.codebase_signals import analyze_codebase
from skilgen.agents.requirements_parser import parse_project_intent
from skilgen.agents.roadmap_planner import build_roadmap_plan
from skilgen.core.config import load_config
from skilgen.core.context import build_codebase_context
from skilgen.core.models import RequirementsContext, SkillSpec
from skilgen.deep_agents_core import run_deep_text


TODAY = date.today().isoformat()


def _signal_bullets(items: list[str], fallback: str, limit: int = 5) -> list[str]:
    if not items:
        return [fallback]
    bullets = [f"Detected: `{item}`" for item in items[:limit]]
    if len(items) > limit:
        bullets.append(f"Detected {len(items) - limit} more matching files elsewhere in the repo.")
    return bullets


def _slug_name(name: str) -> str:
    return name.replace(" ", "-").lower()


def _relative_skill_ref(from_skill_path: str, to_skill_path: str) -> str:
    return os.path.relpath(to_skill_path, Path(from_skill_path).parent).replace("\\", "/")


def _parent_reference_map(context: RequirementsContext, project_root: Path) -> dict[str, str]:
    codebase_context = build_codebase_context(project_root, context)
    return {
        node.domain: node.path
        for node in codebase_context.skill_tree
        if node.parent_skill is None
    }


def _dynamic_parent_specs(context: RequirementsContext, project_root: Path) -> list[SkillSpec]:
    codebase_context = build_codebase_context(project_root, context)
    parent_nodes = [node for node in codebase_context.domain_graph.nodes if node.parent_domain is None and node.skill_path]
    specs: list[SkillSpec] = []
    for node in parent_nodes:
        references = []
        for related in node.related_domains:
            related_node = next((item for item in codebase_context.domain_graph.nodes if item.name == related and item.skill_path), None)
            if related_node is not None and related_node.skill_path != node.skill_path:
                references.append(_relative_skill_ref(node.skill_path, related_node.skill_path))
        for child_name in node.child_domains:
            child_node = next((item for item in codebase_context.domain_graph.nodes if item.name == child_name and item.skill_path), None)
            if child_node is not None:
                references.append(_relative_skill_ref(node.skill_path, child_node.skill_path))
        references = list(dict.fromkeys(references))
        specs.append(
            SkillSpec(
                path=node.skill_path.removeprefix("skills/"),
                name=_slug_name(node.name),
                domain=node.name,
                sub_domain="platform",
                overview=node.summary,
                checks=[f"{{{{project_root}}}}/{Path(item).parts[0]}/" if "/" in item else f"{{{{project_root}}}}/{item}" for item in node.key_files[:3]] or ["{{project_root}}/"],
                patterns=[
                    ("Inferred domain patterns", node.key_patterns or ["Use the inferred project structure before introducing a new top-level convention."]),
                    ("Dynamic topology", ["This parent skill was inferred from the current repo and may expand or contract as the codebase evolves."]),
                ],
                how_to=[
                    "Start from the nearest evidence file in this inferred domain.",
                    "Reuse the current structure before creating a new sibling domain or folder.",
                    "Refresh this parent skill when the planner says the domain topology has changed.",
                ],
                references=references,
            )
        )
    return specs


def _legacy_child_specs(context: RequirementsContext, project_root: Path) -> list[SkillSpec]:
    signals = analyze_codebase(project_root)
    references = _parent_reference_map(context, project_root)
    specs: list[SkillSpec] = []

    if "backend" in references:
        specs.extend(
            [
                SkillSpec(
                    path="backend/api/SKILL.md",
                    name="backend-api",
                    domain="backend",
                    sub_domain="api",
                    overview="Focused guidance for defining or changing API endpoints.",
                    checks=["{{project_root}}/backend/api/", "{{project_root}}/server/routes/", "{{project_root}}/tests/"],
                    patterns=[("Endpoint lifecycle", ["Define the request contract, validate inputs, delegate to services, and map stable responses."])],
                    how_to=[
                        "List all endpoints created or changed by the feature.",
                        "Define request and response contracts.",
                        "Implement the route or controller layer.",
                        "Add tests for every impacted endpoint.",
                    ],
                    references=["../SKILL.md", "../../requirements/SKILL.md"],
                ),
                SkillSpec(
                    path="backend/testing/SKILL.md",
                    name="backend-testing",
                    domain="backend",
                    sub_domain="testing",
                    overview="Verification rules for backend work, especially endpoint coverage.",
                    checks=["{{project_root}}/tests/", "{{project_root}}/backend/tests/", "{{project_root}}/server/tests/"],
                    patterns=[("Endpoint-first verification", ["Test happy paths and failure modes for each impacted endpoint."])],
                    how_to=[
                        "Enumerate every endpoint touched by the feature.",
                        "Add or update tests for success and failure cases.",
                        "Run the relevant test suite before closing the work.",
                    ],
                    references=["../SKILL.md", "../api/SKILL.md"],
                ),
            ]
        )
        if signals.backend_routes:
            specs.append(
                SkillSpec(
                    path="backend/routes/SKILL.md",
                    name="backend-routes",
                    domain="backend",
                    sub_domain="routes",
                    overview="Code-aware guidance for backend routes and handlers already present in the scanned project.",
                    checks=["{{project_root}}/backend/", "{{project_root}}/server/", "{{project_root}}/api/"],
                    patterns=[
                        ("Detected route files", _signal_bullets(signals.backend_routes, "No backend route files were detected.")),
                        ("Route extension pattern", ["Keep request parsing at the edge and move business logic into services."]),
                    ],
                    how_to=[
                        "Start with the closest existing route file from the detected list.",
                        "Trace the handler to the service or use-case layer before making changes.",
                        "Add endpoint tests for every touched success and failure path.",
                    ],
                    references=["../SKILL.md", "../api/SKILL.md", "../testing/SKILL.md"],
                )
            )
        if signals.services:
            specs.append(
                SkillSpec(
                    path="backend/services/SKILL.md",
                    name="backend-services",
                    domain="backend",
                    sub_domain="services",
                    overview="Code-aware guidance for service and use-case modules already present in the scanned project.",
                    checks=["{{project_root}}/backend/services/", "{{project_root}}/services/", "{{project_root}}/src/services/"],
                    patterns=[
                        ("Detected service files", _signal_bullets(signals.services, "No service files were detected.")),
                        ("Service boundary pattern", ["Keep orchestration, validation, and transport concerns separate from business logic."]),
                    ],
                    how_to=[
                        "Start from the closest service file in the detected list.",
                        "Reuse the current service naming and return-shape conventions.",
                        "Add endpoint or unit coverage for the service path you change.",
                    ],
                    references=["../SKILL.md", "../api/SKILL.md", "../testing/SKILL.md"],
                )
            )
        if signals.data_models or signals.persistence_layers:
            specs.append(
                SkillSpec(
                    path="backend/data/SKILL.md",
                    name="backend-data",
                    domain="backend",
                    sub_domain="data",
                    overview="Guidance for models, repositories, persistence layers, and data contracts detected in the project.",
                    checks=["{{project_root}}/models/", "{{project_root}}/repository/", "{{project_root}}/db/", "{{project_root}}/prisma/"],
                    patterns=[
                        ("Detected data model files", _signal_bullets(signals.data_models, "No data model files were detected.")),
                        ("Detected persistence files", _signal_bullets(signals.persistence_layers, "No persistence-layer files were detected.")),
                    ],
                    how_to=[
                        "Start from the nearest model or repository file in the detected list.",
                        "Confirm how data contracts flow between handlers, services, and persistence.",
                        "Keep schema or repository changes aligned with backend route and service guidance.",
                    ],
                    references=["../SKILL.md", "../services/SKILL.md", "../api/SKILL.md"],
                )
            )
        if signals.auth_files:
            specs.append(
                SkillSpec(
                    path="backend/auth/SKILL.md",
                    name="backend-auth",
                    domain="backend",
                    sub_domain="auth",
                    overview="Guidance for authentication, authorization, permission checks, and session handling patterns already present in the repo.",
                    checks=["{{project_root}}/auth/", "{{project_root}}/security/", "{{project_root}}/backend/"],
                    patterns=[("Detected auth files", _signal_bullets(signals.auth_files, "No auth files were detected."))],
                    how_to=[
                        "Trace the current user/session/permission path before making auth changes.",
                        "Validate both authorized and unauthorized backend responses.",
                        "Update endpoint and service tests for every changed auth path.",
                    ],
                    references=["../SKILL.md", "../api/SKILL.md", "../testing/SKILL.md"],
                )
            )
        if signals.background_jobs:
            specs.append(
                SkillSpec(
                    path="backend/jobs/SKILL.md",
                    name="backend-jobs",
                    domain="backend",
                    sub_domain="jobs",
                    overview="Guidance for background workers, cron tasks, queue processors, and offline execution paths detected in the project.",
                    checks=["{{project_root}}/jobs/", "{{project_root}}/workers/", "{{project_root}}/tasks/"],
                    patterns=[("Detected background job files", _signal_bullets(signals.background_jobs, "No background job files were detected."))],
                    how_to=[
                        "Start from the nearest existing job or worker file.",
                        "Identify the shared services or persistence paths the job relies on.",
                        "Add tests or smoke checks for success and failure execution paths.",
                    ],
                    references=["../SKILL.md", "../services/SKILL.md", "../testing/SKILL.md"],
                )
            )

    if "frontend" in references:
        specs.append(
            SkillSpec(
                path="frontend/components/SKILL.md",
                name="frontend-components",
                domain="frontend",
                sub_domain="components",
                overview="Guidance for reusable components and UI composition.",
                checks=["{{project_root}}/frontend/components/", "{{project_root}}/src/components/", "{{project_root}}/app/components/"],
                patterns=[
                    ("Dynamic location awareness", ["Resolve the nearest existing component folder before creating a new one."]),
                    ("Detected component files", _signal_bullets(signals.components, "No reusable component files were detected.")),
                ],
                how_to=[
                    "Find the nearest existing feature or shared component folder.",
                    "Match naming and export conventions for that area.",
                    "Add tests or story coverage if the project uses them.",
                ],
                references=["../SKILL.md", "../../requirements/SKILL.md"],
            )
        )
        if signals.frontend_routes:
            specs.append(
                SkillSpec(
                    path="frontend/routes/SKILL.md",
                    name="frontend-routes",
                    domain="frontend",
                    sub_domain="routes",
                    overview="Code-aware guidance for pages, screens, and route modules already present in the scanned project.",
                    checks=["{{project_root}}/frontend/", "{{project_root}}/src/", "{{project_root}}/app/"],
                    patterns=[
                        ("Detected route files", _signal_bullets(signals.frontend_routes, "No frontend route files were detected.")),
                        ("Route composition", ["Keep route-level data orchestration close to the page and move reusable UI into components."]),
                    ],
                    how_to=[
                        "Start with the closest existing route or page file from the detected list.",
                        "Reuse nearby components and shared state patterns before introducing new abstractions.",
                        "Update tests or route smoke checks if the project already has them.",
                    ],
                    references=["../SKILL.md", "../components/SKILL.md", "../../requirements/SKILL.md"],
                )
            )
        if signals.state_files:
            specs.append(
                SkillSpec(
                    path="frontend/state/SKILL.md",
                    name="frontend-state",
                    domain="frontend",
                    sub_domain="state",
                    overview="Guidance for client state, stores, contexts, reducers, and data-view synchronization patterns detected in the codebase.",
                    checks=["{{project_root}}/src/state/", "{{project_root}}/src/store/", "{{project_root}}/src/context/"],
                    patterns=[("Detected state files", _signal_bullets(signals.state_files, "No state files were detected."))],
                    how_to=[
                        "Start from the nearest existing state/store file in the detected list.",
                        "Trace how routes and components consume that state before changing it.",
                        "Update route or component tests when state behavior changes user-visible flows.",
                    ],
                    references=["../SKILL.md", "../routes/SKILL.md", "../components/SKILL.md", "../../requirements/SKILL.md"],
                )
            )
        if signals.design_system_files:
            specs.append(
                SkillSpec(
                    path="frontend/design-system/SKILL.md",
                    name="frontend-design-system",
                    domain="frontend",
                    sub_domain="design-system",
                    overview="Guidance for themes, tokens, UI kit modules, and design-system conventions already present in the repo.",
                    checks=["{{project_root}}/src/theme/", "{{project_root}}/src/tokens/", "{{project_root}}/src/ui/"],
                    patterns=[("Detected design system files", _signal_bullets(signals.design_system_files, "No design-system files were detected."))],
                    how_to=[
                        "Start from the nearest theme, token, or UI system file in the detected list.",
                        "Trace which components and routes depend on those primitives.",
                        "Update components and tests together when shared design primitives change.",
                    ],
                    references=["../SKILL.md", "../components/SKILL.md", "../../requirements/SKILL.md"],
                )
            )

    plan = build_roadmap_plan(load_config(project_root), parse_project_intent(project_root, context.requirements_path if context.requirements_path.exists() else None))
    if "roadmap" in references:
        phase_paths: list[str] = []
        seen_phases: set[str] = set()
        for step in plan.steps:
            if step.phase in seen_phases:
                continue
            seen_phases.add(step.phase)
            phase_paths.append(f"./{step.phase}/SKILL.md")
            specs.append(
                SkillSpec(
                    path=f"roadmap/{step.phase}/SKILL.md",
                    name=f"roadmap-{step.phase}",
                    domain="roadmap",
                    sub_domain=step.phase,
                    overview=f"Roadmap guidance for {step.title}.",
                    checks=["{{project_root}}/README.md", "{{project_root}}/FEATURES.md", "{{project_root}}/skills/roadmap/"],
                    patterns=[("Roadmap step", [step.description]), ("Status tracking", [f"Current status: {step.status}."])],
                    how_to=[
                        "Read the current step description.",
                        "Use the phase status to decide whether the work is in progress or pending.",
                        "Reflect completed work back into the roadmap tree.",
                    ],
                    references=["../SKILL.md", "../../requirements/SKILL.md"],
                )
            )
    return specs


def build_skill_specs(context: RequirementsContext, output_dir: Path) -> list[SkillSpec]:
    project_root = output_dir.parent
    specs = _dynamic_parent_specs(context, project_root)
    specs.extend(_legacy_child_specs(context, project_root))
    seen: set[str] = set()
    unique: list[SkillSpec] = []
    for spec in specs:
        if spec.path in seen:
            continue
        seen.add(spec.path)
        unique.append(spec)
    return unique


def _render_skill_native(spec: SkillSpec, source_hash: str) -> str:
    depth = len(Path(spec.path).parts)
    traceability_ref = "/".join([".."] * depth + ["TRACEABILITY.md"])
    pattern_sections = []
    for title, bullets in spec.patterns:
        block = [f"### {title}"]
        block.extend(f"- {item}" for item in bullets)
        pattern_sections.append("\n".join(block))

    sections = [
        "---",
        f"name: {spec.name}",
        "version: 0.4.0",
        f"domain: {spec.domain}",
        f"sub_domain: {spec.sub_domain}",
        f"last_updated: {TODAY}",
        "triggered_by: requirements_pipeline",
        f"source_hash: {source_hash}",
        "references:",
        *[f"  - {item}" for item in spec.references],
        "status: active",
        "---",
        "",
        f"# {spec.name.replace('-', ' ').title()} Skill",
        "",
        "## Overview",
        spec.overview,
        "",
        "## Check These Paths First",
        *[f"- {item}" for item in spec.checks],
        "",
        "## Patterns",
        "\n".join(pattern_sections),
        "",
        "## How-To",
        *[f"{index}. {step}" for index, step in enumerate(spec.how_to, start=1)],
        "",
        "## Traceability",
        f"- Generated from requirements source hash: `{source_hash}`",
        f"- Domain path: `{spec.domain}/{spec.sub_domain}`",
        f"- Read `{traceability_ref}` for full requirement-to-output mapping.",
        "- Use the detected file patterns in this skill before creating new structure.",
        "",
        "## References",
        *[f"- {item}" for item in spec.references],
        "",
    ]
    return "\n".join(sections)


def render_skill(spec: SkillSpec, source_hash: str, project_root: Path | str = ".") -> str:
    if spec.path.count("/") >= 2:
        return _render_skill_native(spec, source_hash)
    return run_deep_text(
        "skill guidance synthesis",
        (
            "Write a markdown SKILL.md file with YAML frontmatter for Skilgen. "
            "Use the provided skill spec to generate higher-level reusable guidance while preserving references and "
            "traceability. Optimize for coding agents that need actionable, reusable execution guidance rather than "
            "generic prose. Preserve the intent of the domain, keep the path references stable, and make the How-To "
            "section operational enough that an agent can decide what to inspect, change, and validate next.\n\n"
            f"Skill spec JSON:\n{spec}\nSource hash: {source_hash}"
        ),
        lambda: _render_skill_native(spec, source_hash),
        project_root=project_root,
    )


def render_manifest(specs: list[SkillSpec], source_hash: str) -> str:
    lines = [
        "# Skill Manifest",
        "",
        "This manifest is the entry point for agents discovering the skill tree.",
        "",
        "| Skill Path | Version | Domain | Last Updated | Triggered By | Source Hash |",
        "| --- | --- | --- | --- | --- | --- |",
    ]
    for spec in specs:
        lines.append(f"| `{spec.path}` | `0.4.0` | `{spec.domain}` | `{TODAY}` | `requirements_pipeline` | `{source_hash}` |")
    lines.append("")
    return "\n".join(lines)


def render_graph(specs: list[SkillSpec]) -> str:
    lines = [
        "# Skill Graph",
        "",
        "This file summarizes the generated skill tree and cross references.",
        "",
    ]
    for spec in specs:
        lines.append(f"## {spec.path}")
        lines.append(f"- domain: `{spec.domain}`")
        lines.append(f"- sub_domain: `{spec.sub_domain}`")
        if spec.references:
            lines.append("- references:")
            lines.extend(f"  - `{reference}`" for reference in spec.references)
        else:
            lines.append("- references: none")
        lines.append("")
    return "\n".join(lines)


def render_summary(context: RequirementsContext) -> str:
    lines = ["# Requirements Summary", ""]
    if context.summary:
        lines.extend(f"- {line}" for line in context.summary)
    else:
        lines.append("- No major summary lines were extracted.")
    lines.append("")
    return "\n".join(lines)


def render_domain_summary(title: str, sections: list[tuple[str, list[str]]]) -> str:
    lines = [f"# {title}", ""]
    for heading, items in sections:
        lines.append(f"## {heading}")
        if items:
            lines.extend(f"- {item}" for item in items)
        else:
            lines.append("- No matching files detected.")
        lines.append("")
    return "\n".join(lines)


def _select_specs(specs: list[SkillSpec], selected_domains: set[str]) -> list[SkillSpec]:
    if not selected_domains:
        return specs
    return [spec for spec in specs if spec.domain in selected_domains]


def _dynamic_summary_paths(context: RequirementsContext, output_dir: Path, selected: set[str]) -> list[Path]:
    codebase_context = build_codebase_context(output_dir.parent, context)
    paths: list[Path] = []
    for node in codebase_context.domain_graph.nodes:
        if node.parent_domain is not None or not node.skill_path:
            continue
        if selected and node.name not in selected:
            continue
        skill_rel = Path(node.skill_path.removeprefix("skills/"))
        paths.append(output_dir / skill_rel.parent / "SUMMARY.md")
    return paths


def planned_skill_paths(context: RequirementsContext, output_dir: Path, selected_domains: set[str] | None = None) -> list[Path]:
    selected = selected_domains or set()
    specs = _select_specs(build_skill_specs(context, output_dir), selected)
    planned = [output_dir / spec.path for spec in specs]
    planned.append(output_dir / "MANIFEST.md")
    planned.append(output_dir / "GRAPH.md")
    planned.extend(_dynamic_summary_paths(context, output_dir, selected))
    unique_paths: list[Path] = []
    seen: set[Path] = set()
    for path in planned:
        if path in seen:
            continue
        seen.add(path)
        unique_paths.append(path)
    return unique_paths


def write_skills(context: RequirementsContext, output_dir: Path, selected_domains: set[str] | None = None) -> list[Path]:
    selected = selected_domains or set()
    specs = _select_specs(build_skill_specs(context, output_dir), selected)
    signals = analyze_codebase(output_dir.parent)
    written: list[Path] = []
    for spec in specs:
        target = output_dir / spec.path
        target.parent.mkdir(parents=True, exist_ok=True)
        target.write_text(render_skill(spec, context.source_hash, output_dir.parent), encoding="utf-8")
        written.append(target)

    manifest = output_dir / "MANIFEST.md"
    manifest.write_text(render_manifest(specs, context.source_hash), encoding="utf-8")
    written.append(manifest)

    graph = output_dir / "GRAPH.md"
    graph.write_text(render_graph(specs), encoding="utf-8")
    written.append(graph)

    summary_map: dict[str, list[tuple[str, list[str]]]] = {
        "requirements": [("Planning Inputs", context.summary)],
        "backend": [("Detected Route Files", signals.backend_routes), ("Detected Service Files", signals.services), ("Detected Test Files", signals.tests)],
        "frontend": [("Detected Route Files", signals.frontend_routes), ("Detected Component Files", signals.components), ("Detected Test Files", signals.tests)],
        "design-system": [("Detected Design System Files", signals.design_system_files)],
        "security": [("Detected Auth Files", signals.auth_files)],
        "operations": [("Detected Background Job Files", signals.background_jobs)],
        "data-platform": [("Detected Data Model Files", signals.data_models), ("Detected Persistence Files", signals.persistence_layers)],
        "roadmap": [("Roadmap Context", context.summary)],
    }
    codebase_context = build_codebase_context(output_dir.parent, context)
    for node in codebase_context.domain_graph.nodes:
        if node.parent_domain is not None or not node.skill_path:
            continue
        if selected and node.name not in selected:
            continue
        summary_path = output_dir / Path(node.skill_path.removeprefix("skills/")).parent / "SUMMARY.md"
        summary_path.parent.mkdir(parents=True, exist_ok=True)
        summary_path.write_text(
            render_domain_summary(f"{node.name.replace('-', ' ').title()} Summary", summary_map.get(node.name, [("Key Files", node.key_files)])),
            encoding="utf-8",
        )
        written.append(summary_path)

    if "frontend" in {node.name for node in codebase_context.domain_graph.nodes if node.parent_domain is None} and (
        not selected or "frontend" in selected
    ):
        component_summary = output_dir / "frontend" / "components" / "SUMMARY.md"
        component_summary.parent.mkdir(parents=True, exist_ok=True)
        component_summary.write_text(
            render_domain_summary("Frontend Components Summary", [("Detected Component Files", signals.components)]),
            encoding="utf-8",
        )
        written.append(component_summary)

    if "backend" in {node.name for node in codebase_context.domain_graph.nodes if node.parent_domain is None} and signals.services and (
        not selected or "backend" in selected
    ):
        service_summary = output_dir / "backend" / "services" / "SUMMARY.md"
        service_summary.parent.mkdir(parents=True, exist_ok=True)
        service_summary.write_text(
            render_domain_summary("Backend Services Summary", [("Detected Service Files", signals.services)]),
            encoding="utf-8",
        )
        written.append(service_summary)

    return written



from __future__ import annotations

from pathlib import Path

from skilgen.core.models import SkilgenConfig


DEFAULT_CONFIG = SkilgenConfig(
    include_paths=["."],
    exclude_paths=[".git", "__pycache__", ".venv", "node_modules", ".skilgen"],
    domains_override=[],
    skill_depth=2,
    update_trigger="auto",
    langsmith_project=None,
    model_provider="openai",
    model="gpt-4.1-mini",
    api_key_env="OPENAI_API_KEY",
    model_temperature=None,
    model_max_tokens=None,
    model_retry_attempts=3,
    model_retry_base_delay_seconds=1.0,
    auto_install_external_skills=True,
    external_skills_allowed_trust_levels=["official", "spec", "community", "curated"],
    external_skills_allowlist=[],
    external_skills_denylist=[],
    external_skills_auto_activate=True,
    external_skills_policy_mode="permissive",
    auto_activate_mcp_connectors=True,
    mcp_connectors_require_official_source=True,
    mcp_connectors_require_oauth=True,
    mcp_connector_allowlist=[],
    mcp_connector_denylist=[],
    enterprise_skill_paths=[],
    enterprise_skill_git_urls=[],
)


PROVIDER_DEFAULTS: dict[str, tuple[str, str]] = {
    "openai": ("gpt-4.1-mini", "OPENAI_API_KEY"),
    "anthropic": ("claude-sonnet-4-5", "ANTHROPIC_API_KEY"),
    "gemini": ("gemini-2.5-pro", "GOOGLE_API_KEY"),
    "google": ("gemini-2.5-pro", "GOOGLE_API_KEY"),
    "google_genai": ("gemini-2.5-pro", "GOOGLE_API_KEY"),
    "huggingface": ("meta-llama/Llama-3.1-70B-Instruct", "HUGGINGFACEHUB_API_TOKEN"),
    "hugging_face": ("meta-llama/Llama-3.1-70B-Instruct", "HUGGINGFACEHUB_API_TOKEN"),
    "hf": ("meta-llama/Llama-3.1-70B-Instruct", "HUGGINGFACEHUB_API_TOKEN"),
}


def _parse_scalar(raw: str) -> str | int | float | bool | None:
    value = raw.strip()
    if value in {"", "null", "None"}:
        return None
    if value in {"true", "True"}:
        return True
    if value in {"false", "False"}:
        return False
    if value.isdigit():
        return int(value)
    try:
        if "." in value:
            return float(value)
    except ValueError:
        pass
    if value.startswith(("'", '"')) and value.endswith(("'", '"')):
        return value[1:-1]
    return value


def load_config(project_root: Path) -> SkilgenConfig:
    path = project_root / "skilgen.yml"
    if not path.exists():
        return DEFAULT_CONFIG

    data: dict[str, object] = {
        "include_paths": [],
        "exclude_paths": [],
        "domains_override": [],
        "skill_depth": DEFAULT_CONFIG.skill_depth,
        "update_trigger": DEFAULT_CONFIG.update_trigger,
        "langsmith_project": DEFAULT_CONFIG.langsmith_project,
        "model_provider": DEFAULT_CONFIG.model_provider,
        "model": DEFAULT_CONFIG.model,
        "api_key_env": DEFAULT_CONFIG.api_key_env,
        "model_temperature": DEFAULT_CONFIG.model_temperature,
        "model_max_tokens": DEFAULT_CONFIG.model_max_tokens,
        "model_retry_attempts": DEFAULT_CONFIG.model_retry_attempts,
        "model_retry_base_delay_seconds": DEFAULT_CONFIG.model_retry_base_delay_seconds,
        "auto_install_external_skills": DEFAULT_CONFIG.auto_install_external_skills,
        "external_skills_allowed_trust_levels": list(DEFAULT_CONFIG.external_skills_allowed_trust_levels),
        "external_skills_allowlist": list(DEFAULT_CONFIG.external_skills_allowlist),
        "external_skills_denylist": list(DEFAULT_CONFIG.external_skills_denylist),
        "external_skills_auto_activate": DEFAULT_CONFIG.external_skills_auto_activate,
        "external_skills_policy_mode": DEFAULT_CONFIG.external_skills_policy_mode,
        "auto_activate_mcp_connectors": DEFAULT_CONFIG.auto_activate_mcp_connectors,
        "mcp_connectors_require_official_source": DEFAULT_CONFIG.mcp_connectors_require_official_source,
        "mcp_connectors_require_oauth": DEFAULT_CONFIG.mcp_connectors_require_oauth,
        "mcp_connector_allowlist": list(DEFAULT_CONFIG.mcp_connector_allowlist),
        "mcp_connector_denylist": list(DEFAULT_CONFIG.mcp_connector_denylist),
        "enterprise_skill_paths": list(DEFAULT_CONFIG.enterprise_skill_paths),
        "enterprise_skill_git_urls": list(DEFAULT_CONFIG.enterprise_skill_git_urls),
    }
    current_list: str | None = None

    for line in path.read_text(encoding="utf-8").splitlines():
        stripped = line.strip()
        if not stripped or stripped.startswith("#"):
            continue
        if stripped.startswith("- "):
            if current_list is None:
                continue
            items = data.setdefault(current_list, [])
            if isinstance(items, list):
                items.append(stripped[2:].strip())
            continue
        if ":" not in stripped:
            continue
        key, raw = stripped.split(":", 1)
        key = key.strip()
        raw = raw.strip()
        if raw == "":
            current_list = key
            data.setdefault(key, [])
            continue
        current_list = None
        data[key] = _parse_scalar(raw)

    return SkilgenConfig(
        include_paths=list(data.get("include_paths", [])),
        exclude_paths=list(data.get("exclude_paths", [])),
        domains_override=list(data.get("domains_override", [])),
        skill_depth=int(data.get("skill_depth", DEFAULT_CONFIG.skill_depth)),
        update_trigger=str(data.get("update_trigger", DEFAULT_CONFIG.update_trigger)),
        langsmith_project=data.get("langsmith_project") if isinstance(data.get("langsmith_project"), str) or data.get("langsmith_project") is None else None,
        model_provider=data.get("model_provider") if isinstance(data.get("model_provider"), str) or data.get("model_provider") is None else None,
        model=data.get("model") if isinstance(data.get("model"), str) or data.get("model") is None else None,
        api_key_env=data.get("api_key_env") if isinstance(data.get("api_key_env"), str) or data.get("api_key_env") is None else None,
        model_temperature=float(data.get("model_temperature")) if isinstance(data.get("model_temperature"), (float, int)) else None,
        model_max_tokens=int(data.get("model_max_tokens")) if isinstance(data.get("model_max_tokens"), int) else None,
        model_retry_attempts=int(data.get("model_retry_attempts", DEFAULT_CONFIG.model_retry_attempts)),
        model_retry_base_delay_seconds=float(data.get("model_retry_base_delay_seconds", DEFAULT_CONFIG.model_retry_base_delay_seconds)),
        auto_install_external_skills=bool(data.get("auto_install_external_skills", DEFAULT_CONFIG.auto_install_external_skills)),
        external_skills_allowed_trust_levels=list(data.get("external_skills_allowed_trust_levels", DEFAULT_CONFIG.external_skills_allowed_trust_levels)),
        external_skills_allowlist=list(data.get("external_skills_allowlist", DEFAULT_CONFIG.external_skills_allowlist)),
        external_skills_denylist=list(data.get("external_skills_denylist", DEFAULT_CONFIG.external_skills_denylist)),
        external_skills_auto_activate=bool(data.get("external_skills_auto_activate", DEFAULT_CONFIG.external_skills_auto_activate)),
        external_skills_policy_mode=str(data.get("external_skills_policy_mode", DEFAULT_CONFIG.external_skills_policy_mode)),
        auto_activate_mcp_connectors=bool(data.get("auto_activate_mcp_connectors", DEFAULT_CONFIG.auto_activate_mcp_connectors)),
        mcp_connectors_require_official_source=bool(
            data.get("mcp_connectors_require_official_source", DEFAULT_CONFIG.mcp_connectors_require_official_source)
        ),
        mcp_connectors_require_oauth=bool(data.get("mcp_connectors_require_oauth", DEFAULT_CONFIG.mcp_connectors_require_oauth)),
        mcp_connector_allowlist=list(data.get("mcp_connector_allowlist", DEFAULT_CONFIG.mcp_connector_allowlist)),
        mcp_connector_denylist=list(data.get("mcp_connector_denylist", DEFAULT_CONFIG.mcp_connector_denylist)),
        enterprise_skill_paths=list(data.get("enterprise_skill_paths", DEFAULT_CONFIG.enterprise_skill_paths)),
        enterprise_skill_git_urls=list(data.get("enterprise_skill_git_urls", DEFAULT_CONFIG.enterprise_skill_git_urls)),
    )


def render_default_config(provider: str | None = None) -> str:
    provider_key = provider.strip().lower() if provider else None
    model = ""
    api_key_env = ""
    if provider_key in PROVIDER_DEFAULTS:
        model, api_key_env = PROVIDER_DEFAULTS[provider_key]

    provider_comment = (
        "# Set these to your preferred provider. For example:\n"
        "# openai / gpt-4.1-mini / OPENAI_API_KEY\n"
        "# anthropic / claude-sonnet-4-5 / ANTHROPIC_API_KEY\n"
        "# gemini / gemini-2.5-pro / GOOGLE_API_KEY\n"
        "# huggingface / meta-llama/Llama-3.1-70B-Instruct / HUGGINGFACEHUB_API_TOKEN\n"
    )

    return f"""# Skilgen configuration
include_paths:
  - .
exclude_paths:
  - .git
  - __pycache__
  - .venv
  - node_modules
  - .skilgen
domains_override:
skill_depth: 2
update_trigger: auto
langsmith_project:
{provider_comment}model_provider: {provider_key or ""}
model: {model}
api_key_env: {api_key_env}
model_temperature:
model_max_tokens:
model_retry_attempts: 3
model_retry_base_delay_seconds: 1.0
auto_install_external_skills: true
external_skills_allowed_trust_levels:
  - official
  - spec
  - community
  - curated
external_skills_allowlist:
external_skills_denylist:
external_skills_auto_activate: true
external_skills_policy_mode: permissive
auto_activate_mcp_connectors: true
mcp_connectors_require_official_source: true
mcp_connectors_require_oauth: true
mcp_connector_allowlist:
mcp_connector_denylist:
enterprise_skill_paths:
enterprise_skill_git_urls:
"""



from __future__ import annotations

import json
from concurrent.futures import ThreadPoolExecutor
from dataclasses import dataclass, field
from itertools import count
from pathlib import Path
from threading import Lock
from typing import Callable
from datetime import datetime, timezone


@dataclass
class JobRecord:
    job_id: str
    job_type: str
    status: str
    payload: dict[str, object]
    result: dict[str, object] | None = None
    error: str | None = None
    progress: int = 0
    message: str = "queued"
    created_at: str = field(default_factory=lambda: datetime.now(timezone.utc).isoformat())
    started_at: str | None = None
    finished_at: str | None = None
    cancel_requested: bool = False
    events: list[dict[str, object]] = field(default_factory=list)


class JobCancelledError(RuntimeError):
    pass


_executor = ThreadPoolExecutor(max_workers=4, thread_name_prefix="skilgen-job")
_job_counter = count(1)
_job_lock = Lock()
_jobs: dict[str, JobRecord] = {}


def _job_storage_dir(payload: dict[str, object]) -> Path | None:
    project_root = payload.get("project_root")
    if not isinstance(project_root, str):
        return None
    return Path(project_root).resolve() / ".skilgen" / "jobs"


def _persist_job(job: JobRecord) -> None:
    storage_dir = _job_storage_dir(job.payload)
    if storage_dir is None:
        return
    storage_dir.mkdir(parents=True, exist_ok=True)
    job_path = storage_dir / f"{job.job_id}.json"
    job_path.write_text(json.dumps(job_payload(job), indent=2), encoding="utf-8")


def _load_job_from_disk(job_id: str, project_root: str | Path) -> JobRecord | None:
    job_path = Path(project_root).resolve() / ".skilgen" / "jobs" / f"{job_id}.json"
    if not job_path.exists():
        return None
    payload = json.loads(job_path.read_text(encoding="utf-8"))
    return JobRecord(
        job_id=payload["job_id"],
        job_type=payload["job_type"],
        status=payload["status"],
        payload=payload["payload"],
        result=payload.get("result"),
        error=payload.get("error"),
        progress=int(payload.get("progress", 0)),
        message=str(payload.get("message", "")),
        created_at=payload.get("created_at"),
        started_at=payload.get("started_at"),
        finished_at=payload.get("finished_at"),
        cancel_requested=bool(payload.get("cancel_requested", False)),
        events=list(payload.get("events", [])),
    )


def _next_job_id() -> str:
    return f"job-{next(_job_counter)}"


def update_job(job: JobRecord, *, status: str | None = None, progress: int | None = None, message: str | None = None) -> None:
    with _job_lock:
        if status is not None:
            job.status = status
            if status == "running" and job.started_at is None:
                job.started_at = datetime.now(timezone.utc).isoformat()
            if status in {"completed", "failed", "cancelled"}:
                job.finished_at = datetime.now(timezone.utc).isoformat()
        if progress is not None:
            job.progress = max(0, min(100, progress))
        if message is not None:
            job.message = message
        _persist_job(job)


def append_job_event(job: JobRecord, message: str, progress: int | None = None) -> None:
    event = {
        "timestamp": datetime.now(timezone.utc).isoformat(),
        "message": message,
    }
    if progress is not None:
        event["progress"] = max(0, min(100, progress))
    with _job_lock:
        job.events.append(event)
        _persist_job(job)


def request_cancel(job_id: str, project_root: str | Path | None = None) -> JobRecord | None:
    job = get_job(job_id, project_root)
    if job is None:
        return None
    with _job_lock:
        if job.status in {"completed", "failed", "cancelled"}:
            return job
        job.cancel_requested = True
        if job.status == "queued":
            job.status = "cancelled"
            job.progress = 100
            job.message = "cancelled"
            job.finished_at = datetime.now(timezone.utc).isoformat()
        else:
            job.message = "cancel requested"
        _persist_job(job)
    return job


def submit_job(
    job_type: str,
    payload: dict[str, object],
    fn: Callable[[Callable[[int, str], None]], dict[str, object]],
) -> JobRecord:
    job = JobRecord(job_id=_next_job_id(), job_type=job_type, status="queued", payload=payload)
    with _job_lock:
        _jobs[job.job_id] = job
        _persist_job(job)

    def runner() -> dict[str, object]:
        if job.cancel_requested:
            update_job(job, status="cancelled", progress=100, message="cancelled")
            raise JobCancelledError("job cancelled before start")
        update_job(job, status="running", progress=10, message="running")
        try:
            def report(progress: int, message: str) -> None:
                if job.cancel_requested:
                    update_job(job, status="cancelled", progress=100, message="cancelled")
                    raise JobCancelledError("job cancelled")
                update_job(job, progress=progress, message=message)
                append_job_event(job, message, progress)

            result = fn(report)
            job.result = result
            update_job(job, status="completed", progress=100, message="completed")
            append_job_event(job, "Finished delivery.", 100)
            _persist_job(job)
            return result
        except JobCancelledError:
            job.error = "job cancelled"
            update_job(job, status="cancelled", progress=100, message="cancelled")
            append_job_event(job, "Job cancelled.", 100)
            _persist_job(job)
            raise
        except Exception as exc:  # noqa: BLE001
            job.error = str(exc)
            update_job(job, status="failed", progress=100, message="failed")
            append_job_event(job, f"Job failed: {exc}", 100)
            _persist_job(job)
            raise

    _executor.submit(runner)
    return job


def get_job(job_id: str, project_root: str | Path | None = None) -> JobRecord | None:
    with _job_lock:
        job = _jobs.get(job_id)
    if job is not None:
        return job
    if project_root is None:
        return None
    return _load_job_from_disk(job_id, project_root)


def list_jobs(project_root: str | Path | None = None) -> list[JobRecord]:
    with _job_lock:
        jobs = list(_jobs.values())
    if project_root is None:
        return jobs
    root = Path(project_root).resolve()
    filtered = [job for job in jobs if Path(str(job.payload.get("project_root", ""))).resolve() == root]
    jobs_dir = root / ".skilgen" / "jobs"
    if jobs_dir.exists():
        seen = {job.job_id for job in filtered}
        for path in sorted(jobs_dir.glob("job-*.json")):
            record = _load_job_from_disk(path.stem, root)
            if record is not None and record.job_id not in seen:
                filtered.append(record)
    return sorted(filtered, key=lambda record: record.job_id)


def job_payload(job: JobRecord) -> dict[str, object]:
    return {
        "job_id": job.job_id,
        "job_type": job.job_type,
        "status": job.status,
        "progress": job.progress,
        "message": job.message,
        "created_at": job.created_at,
        "started_at": job.started_at,
        "finished_at": job.finished_at,
        "cancel_requested": job.cancel_requested,
        "events": job.events,
        "payload": job.payload,
        "result": job.result,
        "error": job.error,
    }



from __future__ import annotations

from pathlib import Path

from skilgen.api.service import (
    analyze_payload,
    cancel_job_payload,
    decision_payload,
    create_deliver_job,
    features_payload,
    fingerprint_payload,
    intent_payload,
    job_status_payload,
    jobs_payload,
    map_payload,
    plan_payload,
    preview_payload,
    resume_job_payload,
    report_payload,
    status_payload,
    validate_payload,
)
from skilgen.autoupdate import auto_update_status, ensure_auto_update_worker, stop_auto_update_worker
from skilgen.core.config import render_default_config
from skilgen.delivery import run_delivery, watch_delivery
from skilgen.enterprise_skills import (
    activate_mcp_connector,
    active_enterprise_skills,
    active_mcp_connectors,
    connector_catalog,
    deactivate_mcp_connector,
    generate_enterprise_skill,
    ingest_enterprise_skill,
    list_enterprise_skills,
    recommend_mcp_connectors,
)
from skilgen.external_skills import (
    activate_external_skill,
    active_external_skills,
    detect_external_skill_sources,
    external_skill_lock,
    external_skill_policy,
    deactivate_external_skill,
    export_external_skill_lock,
    get_external_skill,
    import_external_skill_candidates,
    import_external_skill_lock,
    install_external_skill,
    list_external_skills,
    ranked_external_skills,
    remove_external_skill,
    sync_all_external_skills,
    sync_external_skill,
)


def init_project(project_root: str | Path = ".") -> Path:
    root = Path(project_root).resolve()
    root.mkdir(parents=True, exist_ok=True)
    config_path = root / "skilgen.yml"
    if not config_path.exists():
        config_path.write_text(render_default_config(), encoding="utf-8")
    ensure_auto_update_worker(root)
    return config_path


def fingerprint_codebase(project_root: str | Path = ".") -> dict[str, object]:
    return fingerprint_payload(Path(project_root).resolve())


def map_codebase(project_root: str | Path = ".") -> dict[str, object]:
    return map_payload(Path(project_root).resolve())


def analyze_project(project_root: str | Path = ".", requirements: str | Path | None = None) -> dict[str, object]:
    resolved_requirements = Path(requirements).resolve() if requirements is not None else None
    return analyze_payload(Path(project_root).resolve(), resolved_requirements)


def decide_project(project_root: str | Path = ".", requirements: str | Path | None = None) -> dict[str, object]:
    resolved_requirements = Path(requirements).resolve() if requirements is not None else None
    return decision_payload(Path(project_root).resolve(), resolved_requirements)


def parse_intent(requirements: str | Path) -> dict[str, object]:
    return intent_payload(Path(requirements).resolve())


def extract_feature_inventory(requirements: str | Path, project_root: str | Path = ".") -> dict[str, object]:
    return features_payload(Path(requirements).resolve(), Path(project_root).resolve())


def plan_project(requirements: str | Path, project_root: str | Path = ".") -> dict[str, object]:
    return plan_payload(Path(requirements).resolve(), Path(project_root).resolve())


def deliver_project(
    requirements: str | Path,
    project_root: str | Path = ".",
    *,
    targets: tuple[str, ...] = ("docs", "skills"),
    domains: tuple[str, ...] = (),
    dry_run: bool = False,
) -> list[Path]:
    return run_delivery(requirements, project_root, targets=targets, domains=domains, dry_run=dry_run)


def preview_project(
    requirements: str | Path,
    project_root: str | Path = ".",
    *,
    targets: tuple[str, ...] = ("docs", "skills"),
    domains: tuple[str, ...] = (),
) -> dict[str, object]:
    return preview_payload(requirements, project_root, targets=targets, domains=domains)


def update_project(
    requirements: str | Path,
    project_root: str | Path = ".",
    *,
    targets: tuple[str, ...] = ("docs", "skills"),
    domains: tuple[str, ...] = (),
    dry_run: bool = False,
) -> list[Path]:
    return run_delivery(requirements, project_root, targets=targets, domains=domains, dry_run=dry_run)


def watch_project(
    requirements: str | Path,
    project_root: str | Path = ".",
    *,
    targets: tuple[str, ...] = ("docs", "skills"),
    domains: tuple[str, ...] = (),
    interval_seconds: float = 2.0,
    cycles: int = 0,
    once: bool = False,
) -> list[list[Path]]:
    return watch_delivery(
        requirements,
        project_root,
        targets=targets,
        domains=domains,
        interval_seconds=interval_seconds,
        cycles=cycles,
        once=once,
    )


def project_status(project_root: str | Path = ".") -> dict[str, object]:
    return status_payload(Path(project_root).resolve())


def start_auto_update(project_root: str | Path = ".", requirements: str | Path | None = None) -> dict[str, object]:
    resolved_requirements = Path(requirements).resolve() if requirements is not None else None
    return ensure_auto_update_worker(Path(project_root).resolve(), requirements_path=resolved_requirements)


def stop_auto_update(project_root: str | Path = ".") -> dict[str, object]:
    return stop_auto_update_worker(Path(project_root).resolve())


def get_auto_update_status(project_root: str | Path = ".") -> dict[str, object]:
    return auto_update_status(Path(project_root).resolve())


def project_report(project_root: str | Path = ".") -> dict[str, object]:
    return report_payload(Path(project_root).resolve())


def validate_project_outputs(project_root: str | Path = ".") -> dict[str, object]:
    return validate_payload(Path(project_root).resolve())


def start_deliver_job(requirements: str | Path, project_root: str | Path = ".") -> dict[str, object]:
    return create_deliver_job(requirements, project_root)


def cancel_job(job_id: str, project_root: str | Path | None = None) -> dict[str, object]:
    return cancel_job_payload(job_id, Path(project_root).resolve() if project_root is not None else None)


def resume_job(job_id: str, project_root: str | Path | None = None) -> dict[str, object]:
    return resume_job_payload(job_id, Path(project_root).resolve() if project_root is not None else None)


def get_job_status(job_id: str, project_root: str | Path | None = None) -> dict[str, object]:
    return job_status_payload(job_id, Path(project_root).resolve() if project_root is not None else None)


def list_project_jobs(project_root: str | Path | None = None) -> dict[str, object]:
    return jobs_payload(Path(project_root).resolve() if project_root is not None else None)


def list_skill_sources(
    project_root: str | Path = ".",
    *,
    ecosystem: str | None = None,
    search: str | None = None,
) -> dict[str, object]:
    return list_external_skills(Path(project_root).resolve(), ecosystem=ecosystem, search=search)


def detect_skill_sources(project_root: str | Path = ".") -> dict[str, object]:
    return detect_external_skill_sources(Path(project_root).resolve())


def list_active_skill_sources(project_root: str | Path = ".") -> dict[str, object]:
    return {"skills": active_external_skills(Path(project_root).resolve())}


def skill_source_lock(project_root: str | Path = ".") -> dict[str, object]:
    return external_skill_lock(Path(project_root).resolve())


def export_skill_source_lock(project_root: str | Path = ".", output_path: str | Path | None = None) -> dict[str, object]:
    return export_external_skill_lock(project_root=Path(project_root).resolve(), output_path=output_path)


def import_skill_source_lock(
    project_root: str | Path = ".",
    input_path: str | Path = ".",
    *,
    sync_existing: bool = False,
) -> dict[str, object]:
    return import_external_skill_lock(project_root=Path(project_root).resolve(), input_path=input_path, sync_existing=sync_existing)


def import_skill_source_candidates(
    slug: str,
    project_root: str | Path = ".",
    *,
    limit: int = 5,
    active: bool | None = None,
) -> dict[str, object]:
    return import_external_skill_candidates(project_root=Path(project_root).resolve(), slug=slug, limit=limit, active=active)


def rank_skill_sources(project_root: str | Path = ".") -> dict[str, object]:
    return ranked_external_skills(Path(project_root).resolve())


def skill_source_policy(project_root: str | Path = ".") -> dict[str, object]:
    return external_skill_policy(Path(project_root).resolve())


def list_enterprise_skill_sources(project_root: str | Path = ".") -> dict[str, object]:
    return {"skills": list_enterprise_skills(Path(project_root).resolve())}


def ingest_enterprise_skill_source(
    name: str,
    project_root: str | Path = ".",
    *,
    path: str | Path | None = None,
    git_url: str | None = None,
    ref: str | None = None,
    activate: bool | None = None,
    kind: str = "enterprise",
) -> dict[str, object]:
    return {
        "enterprise_skill": ingest_enterprise_skill(
            Path(project_root).resolve(),
            name=name,
            path=path,
            git_url=git_url,
            ref=ref,
            activate=activate,
            kind=kind,
        )
    }


def generate_enterprise_skill_source(
    name: str,
    source_paths: list[str | Path],
    project_root: str | Path = ".",
    *,
    kind: str = "domain",
    activate: bool = True,
) -> dict[str, object]:
    return {
        "enterprise_skill": generate_enterprise_skill(
            Path(project_root).resolve(),
            name=name,
            source_paths=source_paths,
            kind=kind,
            activate=activate,
        )
    }


def list_mcp_connectors(*, system: str | None = None, search: str | None = None) -> dict[str, object]:
    return connector_catalog(system=system, search=search)


def recommend_project_mcp_connectors(project_root: str | Path = ".") -> dict[str, object]:
    return recommend_mcp_connectors(Path(project_root).resolve())


def list_active_mcp_connectors(project_root: str | Path = ".") -> dict[str, object]:
    return {"connectors": active_mcp_connectors(Path(project_root).resolve())}


def activate_project_mcp_connector(slug: str, project_root: str | Path = ".") -> dict[str, object]:
    return {"connector": activate_mcp_connector(Path(project_root).resolve(), slug)}


def deactivate_project_mcp_connector(slug: str, project_root: str | Path = ".") -> dict[str, object]:
    return {"connector": deactivate_mcp_connector(Path(project_root).resolve(), slug)}


def show_skill_source(slug: str, project_root: str | Path = ".") -> dict[str, object]:
    return {"skill": get_external_skill(slug, Path(project_root).resolve())}


def install_skill_source(
    project_root: str | Path = ".",
    *,
    slug: str | None = None,
    git_url: str | None = None,
    name: str | None = None,
    force: bool = False,
    ref: str | None = None,
    active: bool | None = None,
) -> dict[str, object]:
    return {
        "installed_skill": install_external_skill(
            project_root=Path(project_root).resolve(),
            slug=slug,
            git_url=git_url,
            name=name,
            force=force,
            ref=ref,
            active=active,
        )
    }


def sync_skill_source(slug: str, project_root: str | Path = ".") -> dict[str, object]:
    return {"synced_skill": sync_external_skill(project_root=Path(project_root).resolve(), slug=slug)}


def sync_all_skill_sources(project_root: str | Path = ".") -> dict[str, object]:
    return sync_all_external_skills(project_root=Path(project_root).resolve())


def remove_skill_source(slug: str, project_root: str | Path = ".") -> dict[str, object]:
    return {"removed_skill": remove_external_skill(project_root=Path(project_root).resolve(), slug=slug)}


def activate_skill_source(slug: str, project_root: str | Path = ".") -> dict[str, object]:
    return {"activated_skill": activate_external_skill(project_root=Path(project_root).resolve(), slug=slug)}


def deactivate_skill_source(slug: str, project_root: str | Path = ".") -> dict[str, object]:
    return {"deactivated_skill": deactivate_external_skill(project_root=Path(project_root).resolve(), slug=slug)}



import unittest

from skilgen.agents.roadmap_planner import build_roadmap_plan
from skilgen.core.models import ProjectIntent, SkilgenConfig


class RoadmapPlannerTests(unittest.TestCase):
    def test_build_roadmap_plan_includes_endpoint_and_ui_steps(self) -> None:
        config = SkilgenConfig(
            include_paths=["."],
            exclude_paths=[],
            domains_override=[],
            skill_depth=2,
            update_trigger="manual",
            langsmith_project=None,
            model_provider="openai",
            model="gpt-5",
            api_key_env="OPENAI_API_KEY",
            model_retry_attempts=3,
            model_retry_base_delay_seconds=1.0,
        )
        intent = ProjectIntent(
            features=["feature one"],
            domain_concepts=["backend"],
            entities=["SkillVersion"],
            endpoints=["/scan"],
            ui_flows=["dashboard"],
        )
        plan = build_roadmap_plan(config, intent)
        titles = {step.title for step in plan.steps}
        self.assertIn("Add endpoint-aware feature extraction", titles)
        self.assertIn("Add UI flow and route extraction", titles)


if __name__ == "__main__":
    unittest.main()
